package com.sap.test;

/**<b>Class for veryfing SAP logon tickets</b>
 * */
public class SAPTicketVerifier 
{
	
	/**
	 * Array of certificates used for validating the digital signature in the cookie 
	 */
	private java.security.cert.X509Certificate[] certificates = null;
	/**
	 * singleton instance variable
	 */
	private static SAPTicketVerifier instance = null;
	
	/**
	 * singleton 
	 */
	static
	{
		instance = new SAPTicketVerifier();
	}
	
	/**
	 * retreive the singleton instance
	 * @return the singleton of SAPTicketverifier
	 */
	public static SAPTicketVerifier getInstance()
	{
		return instance;		
	}
	
	/**
	 * private constructor
	 */
	private SAPTicketVerifier()  
	{
		java.security.Security.addProvider(new iaik.security.provider.IAIK());

    }
		
	/**
	 * read certificates from a keystore and hold locally
	 * @param filePath full path to the keystore
	 * @param passwd password to access the keystore
	 * @throws TicketVerifierException if input is invalid or an error occurred during reading of keystore
	 */
	public void setCertificatesFromKeyStoreFile(String filePath, char[] passwd)
	{
		if ((isNullOrEmptyString(filePath)) || (passwd.length <= 0))
		{
			throw new TicketVerifierException("SAPTicketVerifier.setCertificatesFromKeyStoreFile called with invalid file path or empty password");
		}
		java.io.InputStream stream = null;
		java.util.ArrayList certs = new java.util.ArrayList();
		try
		{
			stream = new java.io.FileInputStream(filePath); 
			java.security.KeyStore store = java.security.KeyStore.getInstance("JKS","SUN");
			store.load(stream,passwd);
			java.util.Enumeration enu = store.aliases();
			while (enu.hasMoreElements())
			{
			  String alias = (String)enu.nextElement();
			  if (store.isCertificateEntry(alias))
			  {
			  	certs.add(store.getCertificate(alias));
     		  }
			}
			if (certs.size() < 1)
			{
				throw new TicketVerifierException("Keystore "+filePath+" does not contain imported certificates");	
			}
			this.certificates =  (java.security.cert.X509Certificate[])certs.toArray(new java.security.cert.X509Certificate[0]);
		}
		catch (java.io.FileNotFoundException fnfex)
		{
		 throw new TicketVerifierException("File "+filePath+" could not be found in filesystem",fnfex);	
		}
		catch (SecurityException seeex)
		{
		  throw new TicketVerifierException("Access to file "+filePath+" denied",seeex);	
		}
		catch (java.security.KeyStoreException kex)
	    {
		  throw new TicketVerifierException("Internal Error: keystore Type JKS not available",kex);	
	    }
		catch (java.security.cert.CertificateException cex)
		{ 
			throw new TicketVerifierException("Internal Error: Not all certificates in keystore "+filePath+" could be loaded",cex);	
		}
		catch (Exception ex)
		{
			throw new TicketVerifierException("Error",ex);
     	}
		finally
		{
			if (stream != null)
			{
			 try{stream.close();}
			 catch (Exception ex){throw new TicketVerifierException("Error",ex);}
			} 
		}
  	}
	
	/**
	 * Verify a base64 encoded logon ticket and return Ticket Data 
	 * @param base64Value the base64 encoded cookie value
	 * @return SAPTicketInfo containing ticket data
	 * @throws TicketVerifierException if input is invalid, no certificates have been loaded, 
	 * ticket can not be verified or has expired
	 */
	public SAPTicketInfo verifyTicket(String base64Value)
	{
		if (isNullOrEmptyString(base64Value))
		{
			throw new TicketVerifierException("SAPTicketVerifier.verifyTicket called with input Base64 value of nul or empty");	
		}
		SAPTicketInfo info = null;
		if (this.certificates == null)
		{
			throw new TicketVerifierException("No certificates present");
		}
		try
		{
		  com.sap.security.core.ticket.imp.Ticket ticket = new com.sap.security.core.ticket.imp.Ticket();
		  //ticket.debug = true;
		  ticket.setCertificates(this.certificates);
		  ticket.setTicket(base64Value);
		  ticket.verify();

                  if (ticket.isValid()) {
		  	info = new SAPTicketInfo(ticket);
	                return info;								  
		  } 
		  else
		  {
			throw new TicketVerifierException("Ticket is invalid");
		  }
		}
		catch (Exception ex)
		{
		  throw new TicketVerifierException("Error in verifying ticket "+ex.getMessage(),ex);
		}
		
	}
	
	/**
	 * check if a tring value is empty or null
	 * @param value the value to be checked 
	 * @return true is input is empty or null
	 */
	public static boolean isNullOrEmptyString(String value)
	{
			return ((value == null) || (value.trim().length() == 0));
	}
	
	/**
	 * Class containing the data of a logon ticket
	 */
	public static class SAPTicketInfo
	{
        	/**
        	 * the user name of the logged on user
        	 */		
		private String user;
		/**
		 * the name of the ticket issuing SAP system 
		 */
		private String issuingSystem;
		/**
		 * the client of the issuing SAP system 
		 */
		private String issuingSystemClient;
		/**
		 * the expiration date of the ticket
		 */
		private java.util.Calendar expirationDate;
		/**
		 * the code page used by the issuing SAP system 
		 */
		private String sapCodePage;

		private String portalUser;

		private com.sap.security.core.ticket.imp.Ticket ticket;
	
		/**
		 * Constructor 
		 * @param ticket
		 */
		public SAPTicketInfo(com.sap.security.core.ticket.imp.Ticket ticket)
        	{
			this.ticket = ticket;

	                try 
        	        {
        		    this.user = ticket.getUser();
	        	    this.issuingSystem = ticket.getSystemID();
        		    this.issuingSystemClient = ticket.getSystemClient();
        		    this.expirationDate = ticket.getExpirationDate();
	        	    this.sapCodePage = ticket.getCodepage();	                  
	
        	            // Get portal user
                 	    com.sap.security.api.ticket.InfoUnit iu = ticket.getInfoUnit (0x20);
	      
        		    if (iu!=null) {
                	    	// portal user is in UTF8 encoding
	                    	String portal_user = iu.getString ("UTF8");
        	            	if (0!=portal_user.indexOf ("portal:"))
                	            this.portalUser="Invalid info unit.";
	                    	else {
        	                    this.portalUser = portal_user.substring (7);
                	    	}
	                    }
	                }
        	        catch (Exception ex)
			{
			    System.out.println(ex);
			}
	        }
        
        	/**
	         * get user name from ticket
        	 * @return user name
	         */
        	public String getUser()
	        {
        		return this.user;
	        }
        
		/**
		 * get name of ticket issuing SAP system
		 * @return Systen name 
		 */
        	public String getSystem()
	        {
        		return this.issuingSystem;
	        }
        
		/**
		 * get client of ticket issuing SAP system
		 * @return Systen client  
		 */
        	public String getClient()
	        {
        		return this.issuingSystemClient;
	        }
        
		/**
		 * get expiration date of ticket 
		 * @return Expiration date  
		 */
        	public java.util.Calendar getExpirationDate()
	        {
        		return this.expirationDate;
        	}
	
		/**
		 * get expiration date of ticket 
		 * @return Expiration date  
		 */
		public com.sap.security.core.ticket.imp.Ticket getTicket()
		{
			return this.ticket;	
		}

		public boolean isValid()
		{
			return this.ticket.isValid();
		}

		public String getPortalUser()
		{
			return this.portalUser;
		}

		public String validUntil()
		{
			java.text.SimpleDateFormat sdf1 = new java.text.SimpleDateFormat("yyyy-MM-dd");
			java.text.SimpleDateFormat sdf2 = new java.text.SimpleDateFormat("HH:mm:ssZ");
			java.util.Date d = this.getExpirationDate().getTime();

			return sdf1.format(d)+"T"+sdf2.format(d);
		}
	}

}
