package com.sap.test;
import java.io.IOException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import java.net.URLDecoder;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**<b>Servlet class used to verify SAP logon tickets issued by a SAP J2EE engine.</b> <br>
 * Prints the name of the user who logged on to the SAP J2EE engine 
 * 
 * */
public class TicketVerifierServlet extends HttpServlet 
{

    /**
     * Name of cookie issued by SAP J2EE engine
     */
	private static final String MYSAPCOOKIENAME ="MYSAPSSO2";

	/**
	 * Name of init parameter holding the complete path to the keystore file
	 * e.g C:/keystore/portal.store
	 */
	private static final String KEYSTORE_PATH ="KEYSTOREPATH";
	/**
	 * Name of init parameter holding the password to the keystore
	 * Note: in a productive environment the password should be read from a 
	 * secure storage
	 **/
	private static final String KEYSTORE_PASSWORD ="KEYSTOREPASSWORD";
	
	/**
	 * Read path to keystore file and password from init parameters in web.xml file. Initialize Verifier clas with certificates contained in keystore 
	 * @param cfg ServletConfig
	 * @throws TicketVerifierException if error occurred during reading certificates from keystore
	 * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
	 */
	public void init(ServletConfig cfg) throws ServletException 
	{
		super.init(cfg);
		String keystore = cfg.getInitParameter(KEYSTORE_PATH);
		if (SAPTicketVerifier.isNullOrEmptyString(keystore))
		{
			throw new ServletException("No init parameter "+KEYSTORE_PATH+" " +
				                       " specified for a path to the keystore");	
		}
		if (SAPTicketVerifier.isNullOrEmptyString(cfg.getInitParameter(KEYSTORE_PASSWORD)))
		{
			throw new ServletException("No init parameter "+KEYSTORE_PASSWORD+
									   " specified for the password to open the keystore "+keystore);	
		}
		
		char[] passwd = cfg.getInitParameter(KEYSTORE_PASSWORD).toCharArray();
		SAPTicketVerifier SAPVerifier = SAPTicketVerifier.getInstance();
		SAPVerifier.setCertificatesFromKeyStoreFile(keystore,passwd);
		java.util.Arrays.fill(passwd,' ');
	}

	protected void doGet(
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException, IOException 
		{
		 this.doPost(request,response);
	    }

    /**
     * process request 
     */
	protected void doPost(
		HttpServletRequest request,
		HttpServletResponse response)
		throws ServletException, IOException 
		{
			
		  String info = this.getInfoFromRequest(request);
		  java.io.PrintWriter out = response.getWriter();
		  out.println(info);
		 
	    }
	  /**
	   * Extract info from request. The verification method will use 
	   * an SAPTicketVerifier.SAPTicketInfo instance in case you need aditional information
	   * @param request the request
	   * @return the info of the logged on to the J2EE engine
	   */
	  public String getInfoFromRequest(HttpServletRequest request)
	  {
		javax.servlet.http.Cookie ssoCookie = this.getSSOCookie(request);
		String base64Value = this.getCookieValueAsBase64(ssoCookie);
		SAPTicketVerifier verifier = SAPTicketVerifier.getInstance();
		SAPTicketVerifier.SAPTicketInfo info = verifier.verifyTicket(base64Value);
		if (info == null)
		{
			throw new TicketVerifierException("Ticket is invalid");
		}
		return info.getPortalUser() + ";" 
				+ info.getUser() + ";" 
				+ info.getSystem() + ";" 
				+ info.getClient() + ";" 
				+ info.isValid() + ";" 
				+ info.validUntil() + ";" ;
	  }
	   
	  /**
	   * Extract user name from request. The verification method will use 
	   * an SAPTicketVerifier.SAPTicketInfo instance in case you need aditional information
	   * @param request the request
	   * @return the name of the user logged on to the J2EE engine
	   */
	  public String getUserFromRequest(HttpServletRequest request)
	  {
		javax.servlet.http.Cookie ssoCookie = this.getSSOCookie(request);
		String base64Value = this.getCookieValueAsBase64(ssoCookie);
		SAPTicketVerifier verifier = SAPTicketVerifier.getInstance();
		SAPTicketVerifier.SAPTicketInfo info = verifier.verifyTicket(base64Value);
		if (info == null)
		{
			throw new TicketVerifierException("Ticket is invalid");
		}
		return info.getUser();
	  }
	  
	  /**
	   * Extract the value of the MYSAPSSO2 cookie as base64 encoded String
	   * @param cook the cookie containing the SSO information
	   * @return the base64Value of the cookie content
	   * @throws TicketVerifierException if input is null or URLDecoding could not be performed 
	   */
	  private String getCookieValueAsBase64(javax.servlet.http.Cookie cook)
	  {
	  	if (cook == null)
	  	{
			throw new TicketVerifierException("TicketVerifierServlet.getCookieValueAsBase64 called with input of null");
	  	}
	  	String base64Value = null;
	  	String cookieValue = cook.getValue();
	  	if (!SAPTicketVerifier.isNullOrEmptyString(cookieValue))
	  	{
		 base64Value = cookieValue;
		 /*
	  	 try
	  	 { 
			base64Value = URLDecoder.decode(cookieValue,"UTF-8");
	  	 }
	  	 catch (java.io.UnsupportedEncodingException useex)
	  	 {
	  	 	throw new TicketVerifierException("UTF-8 encoding not supported",useex);
	  	 }
		 */
     	        }
	    return base64Value;	
	  }
	  
	  /**
	   * Extract the MYSAPSSO2 Cookie form the request
	   * @param request the request
	   * @return the MYSAPSSO2 Cookie 
	   * @throws TicketVerifierException if request is null or does not contain a cookie named MYSAPSSO2
	   */
	  private javax.servlet.http.Cookie getSSOCookie(HttpServletRequest request) throws TicketVerifierException
	  {
	  	if (request == null)
	  	{
			throw new TicketVerifierException("method getSSOCooie called with input request of null");
	  	}
		javax.servlet.http.Cookie SAPCookie = null;
		javax.servlet.http.Cookie[] cookies = request.getCookies();
		if (cookies == null)
		{
			throw new TicketVerifierException("Request does not contain cookies");
		}
		else
		{
			for (int i = 0; i < cookies.length; i++) 
			{
				javax.servlet.http.Cookie cook = cookies[i];
				if (cook.getName().trim().equals(MYSAPCOOKIENAME))
			    {
					SAPCookie = cook;
					break;
			   }
		    }
		}
		if (SAPCookie == null)
		{
		  throw new TicketVerifierException("Request does not contain a cookie named "+MYSAPCOOKIENAME);	
		}
		return SAPCookie;
	  }
		    


      
      
      
}

